/*
 *  Gourad.cpp - 	 Gourad effect
 *               Written by Tinic Urou
 *               Public domain
 */

#include <AppKit.h>
#include <InterfaceKit.h>
#include <KernelKit.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "test.h"


// Types
typedef unsigned char 	UBYTE;
typedef unsigned short 	UWORD;
typedef unsigned long 	ULONG;
typedef char 			BYTE;
typedef short 			WORD;
typedef long 			LONG;

// Constants

const 	ULONG 			APP_SIGNATURE = '????';
const 	LONG 			DISPLAY_X = 320;
const 	LONG 			DISPLAY_Y = 240;

// Globals

		BBitmap			*my_bitmap;
		uchar			colors[256],*bits;


// Window/Application messages

const 	LONG			MSG_REDRAW = 1;

class 					GouradWindow;

// Application object
class Gourad : public BApplication 
{
	public:
						Gourad();
			void 		ReadyToRun(void);
			bool 		QuitRequested(void);
			void 		AboutRequested(void);

	private:
	
			GouradWindow 	*the_window;
			BBitmap 	*the_bitmap;
};

// Window object
class GouradWindow : public BWindow 
{
	public:
	
						GouradWindow(short width, short height, BBitmap *bitmap);
			bool 		QuitRequested(void);
			void 		MessageReceived(BMessage *msg);

	private:
	
			void		recalc(void);

			BBitmap 	*the_bitmap;
			BView		*main_view;	
			ULONG		xmod;
};

//------------------------ fixed poshort math code -------------------------

typedef long Fixedposhort;

#define Int2Fixed(a)  (Fixedposhort)((Fixedposhort)(a) << 16)
#define Fixed2Int(a)  (short)((a) >> 16)
#define Float2Fixed(a)  ((Fixedposhort)((a) * 65536.0))
#define Fixed2Float(a)  ((a) / 65536.0)

inline Fixedposhort FixedMul(Fixedposhort f1,Fixedposhort f2);

inline Fixedposhort FixedMul(Fixedposhort f1,Fixedposhort f2)
{
	float i1=Fixed2Float(f1);
	float i2=Fixed2Float(f2);
	
	return((Fixedposhort)Float2Fixed(i1*i2));
};

inline Fixedposhort FixedDiv(Fixedposhort f1,Fixedposhort f2);

inline Fixedposhort FixedDiv(Fixedposhort f1,Fixedposhort f2)
{
	float i1=Fixed2Float(f1);
	float i2=Fixed2Float(f2);
	
	return((Fixedposhort)Float2Fixed(i1/i2));
};

//--------------------------------------------------------------------------

// maximum # of degrees for sin and cos tables
#define MAXDEGREES 720

// distance from users eye to screen surface in pixels
#define EYE_DISTANCE Int2Fixed(256)

//---------------------------- data structures ---------------------------

char filename[80]; // filename of object to load/save

Fixedposhort cosine[MAXDEGREES];  // cosine lookup table
Fixedposhort sine[MAXDEGREES];    // sine lookup table

// these are for the gouraud shading routines
short edge_l[240];
short edge_r[240];
char color_l[240];
char color_r[240];

#define MAXVERTICES 300
#define MAXPOLYGONS 300
#define MAXPOLYVERT 20

// structure of one vertex
typedef struct
{
	Fixedposhort Vox,Voy,Voz; // vertex original coordinates
	Fixedposhort Vwx,Vwy,Vwz; // vertex working  coordinates
	short Vsx,Vsy,Vsz; // vertex screen coordinates
	Fixedposhort Nox,Noy,Noz; // normal's original coordinates
	Fixedposhort Nwx,Nwy,Nwz; // normal's working coordinates
	short shadedcolor; // color of vertex used for gouraud shading
} VertexTYPE;

// structure of one polygon
typedef struct
{
	short NumOfVertices; // number of vertices that make up the polygon
	short Vertex[MAXPOLYVERT]; // vertex indices in counter clockwise order
	short zcenter; // center z coordinate (used for polygon sorting)
	short color; // color group of polygon (before shading)
	short shadedcolor; // actual color value of polygon after shading
} PolygonTYPE;

// object structure
typedef struct
{
	Fixedposhort Ox,Oy,Oz; // coordinates of object's origin
	short Ax,Ay,Az; // object's rotation angles
	short NumOfVertices; // number of vertices in object
	VertexTYPE Vertex[MAXVERTICES]; // all vertices in object
	short NumOfPolygons; // number of polygons in object
	PolygonTYPE Polygon[MAXPOLYGONS]; // all polygons in object
} ObjectTYPE;

ObjectTYPE Object; // one object

short NumOfSortedPolygons; // number of sorted visible polygons
short PolygonOrderList[MAXPOLYGONS]; // list of polygon indices for qsorting

// structure of light source
typedef struct
{
	Fixedposhort x,y,z; // coodinates of light source
	Fixedposhort wx,wy,wz; // working (shortermediate) coordinates
	short ax,ay,az; // rotation angles
} LightSourceTYPE;

LightSourceTYPE LightSource; // one light source


short eboxtop1; // erase box top coordinate (current frame)
short eboxbottom1; // erase box bottom coordinate (current frame)
short eboxleft1; // erase box left coordinate (current frame)
short eboxright1; // erase box right coordinate (current frame)
short eboxtop2; // erase box top coordinate (one frame ago)
short eboxbottom2; // erase box bottom coordinate (one frame ago)
short eboxleft2; // erase box left coordinate (one frame ago)
short eboxright2; // erase box right coordinate (one frame ago)
short eboxtop3; // erase box top coordinate (two frames ago)
short eboxbottom3; // erase box bottom coordinate (two frames ago)
short eboxleft3; // erase box left coordinate (two frames ago)
short eboxright3; // erase box right coordinate (two frames ago)

// dot product, used for light source shading
Fixedposhort DotProduct;


//------------------------ function prototypes ---------------------------

void CreateLookupTables(void);
void RotateObject(void);
void LoadObject(void);
void SortPolygons(void);
int ComparePolygons(const void *a, const void *b);
void DrawPolygons(void);
void CalculateColor(void);
void SetupLightSource(void);
void RotateNormals(void);
void RotateLightSource(void);
void CalculateNormals(void);
void GTriangle(float,float,float, float,float,float, float,float,float,short);
void ScanEdge(float,float,float,float,float,float);
void MakePolygonList(void);

//---------------- code to create sin and cos lookup tables --------------

void CreateLookupTables(void)
{
	short i;
	for(i=0; i<MAXDEGREES; i++)
	{
		cosine[i]=Float2Fixed(cos((float)i*360/MAXDEGREES * 3.14159265 / 180.0));
		sine[i]  =Float2Fixed(sin((float)i*360/MAXDEGREES * 3.14159265 / 180.0));
	}
}


//------------------------------- rotate object -----------------------


void RotateObject(void)
{
	short i;
	Fixedposhort nx,ny,nz,cosxangle,sinxangle,cosyangle,sinyangle,coszangle,sinzangle;

	VertexTYPE *vert; // poshorter to a vertex structure

	// get the sine and cosine angles to save time from table lookup
	sinxangle=sine[Object.Ax];
	cosxangle=cosine[Object.Ax];
	sinyangle=sine[Object.Ay];
	cosyangle=cosine[Object.Ay];
	sinzangle=sine[Object.Az];
	coszangle=cosine[Object.Az];

	// initialize the erase-box info to the extreme values
	eboxtop1=240;
	eboxleft1=320;
	eboxright1=0;
	eboxbottom1=0;

	// loop through all vertices in object
	for(i=0;i<Object.NumOfVertices;i++)
	{
		vert=&Object.Vertex[i];

		// rotate around the x-axis
		vert->Vwz=FixedMul(vert->Voy,cosxangle) - FixedMul(vert->Voz,sinxangle);
		vert->Vwy=FixedMul(vert->Voy,sinxangle) + FixedMul(vert->Voz,cosxangle);
		vert->Vwx=vert->Vox;

		// rotate around the y-axis
		nx=FixedMul(vert->Vwx,cosyangle) - FixedMul(vert->Vwz,sinyangle);
		nz=FixedMul(vert->Vwx,sinyangle) + FixedMul(vert->Vwz,cosyangle);
		vert->Vwx=nx;
		vert->Vwz=nz;

		// rotate around the z-axis
		nx=FixedMul(vert->Vwx,coszangle) - FixedMul(vert->Vwy,sinzangle);
		ny=FixedMul(vert->Vwx,sinzangle) + FixedMul(vert->Vwy,coszangle);
		vert->Vwx=nx;
		vert->Vwy=ny;

		// project the 3-D coordinates to screen coordinates
		vert->Vsx=Fixed2Int(FixedMul(FixedDiv(vert->Vwx+Object.Ox,vert->Vwz+Object.Oz),EYE_DISTANCE)) + 160;
		vert->Vsy=Fixed2Int(FixedMul(FixedDiv(vert->Vwy+Object.Oy,vert->Vwz+Object.Oz),EYE_DISTANCE)) + 120;
		vert->Vsz=Fixed2Int(vert->Vwz+Object.Oz);

		// while we are at it, update the erase-box information
		if(vert->Vsx < eboxleft1) eboxleft1 = vert->Vsx;
		if(vert->Vsx > eboxright1) eboxright1 = vert->Vsx;
		if(vert->Vsy < eboxtop1) eboxtop1 = vert->Vsy;
		if(vert->Vsy > eboxbottom1) eboxbottom1 = vert->Vsy;
	}

	// clip the erase box
	if(eboxleft1<0) eboxleft1=0;
	if(eboxtop1<0) eboxtop1=0;
	if(eboxright1>319) eboxright1=319;
	if(eboxbottom1>239) eboxbottom1=239;
}


//----------------------- load object-------------------------------------

void swab(register short *src);

void swab(register short *src)
{
	unsigned short tmp;	
	tmp=*src; *src=((0xFF00&tmp)>>8)+((0xFF&tmp)<<8);
}

void LoadObject(void)
{
	FILE *file;
	char filename[80];
	short i,j,temp,result;

	VertexTYPE *vert; // poshorter to a vertex structure

	// open the file to write to it
	if ((file = fopen("flag.v10","rb")) == NULL)
	{
	}
	else
	{
		// okay file is ready to read data from it

		// read number of dots in file
		fread(&Object.NumOfVertices,sizeof(short),1,file);
		
		swab(&Object.NumOfVertices);
		
		// read in all of the object's vertices
		
		for(i=0;i < Object.NumOfVertices; i++)
		{
			fread(&temp,sizeof(short),1,file);
			swab(&temp);
			Object.Vertex[i].Vox=Int2Fixed(temp);
			fread(&temp,sizeof(short),1,file);
			swab(&temp);
			Object.Vertex[i].Voy=Int2Fixed(temp);
			fread(&temp,sizeof(short),1,file);
			swab(&temp);
			Object.Vertex[i].Voz=Int2Fixed(temp);
		}

		// read in the number of polygons
		fread(&Object.NumOfPolygons,sizeof(short),1,file);
		swab(&Object.NumOfPolygons);
		// read in the information for each polygon
		for(i=0; i < Object.NumOfPolygons; i++)
		{
			// read in the number of vertices in polygon
			fread(&Object.Polygon[i].NumOfVertices,sizeof(short),1,file);
			swab(&Object.Polygon[i].NumOfVertices);
			// read in the polygon's vertices (already in counter clockwise order)
			for(j=0; j< Object.Polygon[i].NumOfVertices; j++)
			{
				fread(&Object.Polygon[i].Vertex[j],sizeof(short),1,file);
				swab(&Object.Polygon[i].Vertex[j]);
			}
			// read the color of the polygon
			fread(&Object.Polygon[i].color,sizeof(short),1,file);
			swab(&Object.Polygon[i].color);
		}

		// we're finished, close the file
		fclose(file);
	}
}


//-------------------------- make visible polygon list ---------------------

void MakePolygonList(void)
{
	short i,j,k;

	VertexTYPE *v0,*v1,*v2;

	j=0;
	for(i=0;i<Object.NumOfPolygons;i++)
	{
		v0=&Object.Vertex[Object.Polygon[i].Vertex[0]];
		v1=&Object.Vertex[Object.Polygon[i].Vertex[1]];
		v2=&Object.Vertex[Object.Polygon[i].Vertex[2]];

		// if expression results in a negative then polygon is visible
		if( ((v1->Vsx - v0->Vsx) * (v2->Vsy - v0->Vsy) - (v1->Vsy - v0->Vsy) * (v2->Vsx - v0->Vsx)) < 0 )
		{
			PolygonOrderList[j++]=i;
		}
	}
	NumOfSortedPolygons=j;
}

//------------------------------------ sort polygons -----------------------


void SortPolygons(void)
{
	short i,j;
	short maxz,minz;
	short temp;
	PolygonTYPE *poly;

	// first find the distance of each polygon (from midposhort of max & min z's)
	for(i=0;i<Object.NumOfPolygons;i++)
	{

		poly=&Object.Polygon[i];
		minz=65535; // set to extreme values
		maxz=-65536; // set to extreme values
		for(j=0;j<poly->NumOfVertices;j++)
		{
			if(Object.Vertex[poly->Vertex[j]].Vsz < minz)
			{
				minz=Object.Vertex[poly->Vertex[j]].Vsz;
			}
			if(Object.Vertex[poly->Vertex[j]].Vsz > maxz)
			{
				maxz=Object.Vertex[poly->Vertex[j]].Vsz;
			}
		}
		// now calculate the distance
		poly->zcenter=(maxz+minz)>>1;
	}

	// qsort the polygons
	qsort((void *)PolygonOrderList,NumOfSortedPolygons,sizeof(PolygonOrderList[0]),ComparePolygons);
}

//-------------------------- compare zcenter (for qsort) -------------------

int ComparePolygons(const void *a, const void *b)
{
	if( Object.Polygon[*(short *)a].zcenter < Object.Polygon[*(short *)b].zcenter )
	{
		return -1;
	}
	else if( Object.Polygon[*(short *)a].zcenter > Object.Polygon[*(short *)b].zcenter )
	{
		return +1;
	}
	else
	{
		return 0;
	}
}

//------------------------------------ draw polygons -----------------------


void DrawPolygons(void)
{
	short i,j;

//	x_rect_fill(eboxleft3,eboxtop3,eboxright3+1,eboxbottom3+1,HiddenPageOffs,0);
	
	memset(bits,0,DISPLAY_X*DISPLAY_Y);

	for(i=0;i<NumOfSortedPolygons;i++)
	{
		// loop through all vertices in polygon and draw triangular pieces
		for(j=0; j < Object.Polygon[PolygonOrderList[i]].NumOfVertices-2;j++)
		{
			GTriangle((float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[0]].Vsx,
								(float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[0]].Vsy,
								(float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[0]].shadedcolor,
								(float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[j+1]].Vsx,
								(float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[j+1]].Vsy,
								(float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[j+1]].shadedcolor,
								(float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[j+2]].Vsx,
								(float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[j+2]].Vsy,
								(float)Object.Vertex[Object.Polygon[PolygonOrderList[i]].Vertex[j+2]].shadedcolor,
								0);
		}
	}

	// update the older erase box information
	eboxtop3 = eboxtop2; eboxtop2 = eboxtop1;
	eboxleft3 = eboxleft2; eboxleft2 = eboxleft1;
	eboxright3 = eboxright2; eboxright2 = eboxright1;
	eboxbottom3 = eboxbottom2; eboxbottom2 = eboxbottom1;

}


//---------------------------- calculate normals -------------------------


void CalculateNormals(void)
{
	double xlen,ylen,zlen,length;

	//This will calculate the normals of the vertices for gouraud shading

	for(short i=0;i < Object.NumOfVertices; i++)
	{

		// calculate perpendicular via the vertex and the vertex*2
		// ie. we are scaling the vertices and use them as the new vertices
		// for gouraud shading.  This is not quite correct though, because
		// you wouldn't be accounting for polygons that face the object's
		// center.
		xlen = Fixed2Float(FixedMul(Object.Vertex[i].Vox,Int2Fixed(2)));
		ylen = Fixed2Float(FixedMul(Object.Vertex[i].Voy,Int2Fixed(2)));
		zlen = Fixed2Float(FixedMul(Object.Vertex[i].Voz,Int2Fixed(2)));

		// calculate the length of the normal
		length = sqrt(xlen*xlen + ylen*ylen + zlen*zlen);

		// scale it to a unit normal
		Object.Vertex[i].Nox = Float2Fixed(xlen / length);
		Object.Vertex[i].Noy = Float2Fixed(ylen / length);
		Object.Vertex[i].Noz = Float2Fixed(zlen / length);
	}
}


//---------------------------- calculate color ---------------------------

void CalculateColor(void)
{
	short i;
	short color;

	for(i=0; i < Object.NumOfVertices; i++)
	{
		DotProduct=FixedMul(Object.Vertex[i].Nwx , LightSource.wx) +
							 FixedMul(Object.Vertex[i].Nwy , LightSource.wy) +
							 FixedMul(Object.Vertex[i].Nwz , LightSource.wz);

		Object.Vertex[i].shadedcolor = 56 + Fixed2Int(FixedMul(DotProduct,Int2Fixed(40)));
	}
}


//---------------------------- rotate normals -----------------------------


void RotateNormals(void)
{
	short i;
	Fixedposhort nx,ny,nz,cosxangle,sinxangle,cosyangle,sinyangle,coszangle,sinzangle;
	VertexTYPE *vert; // poshorter to a vertex structure

	// get sine and cosine angles to save time from table lookup in inner loop
	sinxangle=sine[Object.Ax];
	cosxangle=cosine[Object.Ax];
	sinyangle=sine[Object.Ay];
	cosyangle=cosine[Object.Ay];
	sinzangle=sine[Object.Az];
	coszangle=cosine[Object.Az];

	for(i=0;i<Object.NumOfVertices;i++)
	{
		vert=&Object.Vertex[i];

		// rotate around the x-axis
		vert->Nwz=FixedMul(vert->Noy , cosxangle) - FixedMul(vert->Noz , sinxangle);
		vert->Nwy=FixedMul(vert->Noy , sinxangle) + FixedMul(vert->Noz , cosxangle);
		vert->Nwx=vert->Nox;

		// rotate around the y-axis
		nx=FixedMul(vert->Nwx , cosyangle) - FixedMul(vert->Nwz , sinyangle);
		nz=FixedMul(vert->Nwx , sinyangle) + FixedMul(vert->Nwz , cosyangle);
		vert->Nwx=nx;
		vert->Nwz=nz;

		// rotate around the z-axis
		nx=FixedMul(vert->Nwx , coszangle) - FixedMul(vert->Nwy , sinzangle);
		ny=FixedMul(vert->Nwx , sinzangle) + FixedMul(vert->Nwy , coszangle);

		// reverse the direction of the normals for lightsource shading
		vert->Nwx=-nx;
		vert->Nwy=-ny;
		vert->Nwz=-nz;
	}
}


//--------------------- set up light source coordinates ------------------


void SetupLightSource(void)
{
	double xlen,ylen,zlen,length;

	// assign the delault light source position
	xlen=0;
	ylen=0;
	zlen=10;

	// calculate the length of the vector (light source to 0,0,0)
	length = sqrt(xlen*xlen + ylen*ylen + zlen*zlen);

	// scale it to a unit vector
	LightSource.x = Float2Fixed(xlen/length);
	LightSource.y = Float2Fixed(ylen/length);
	LightSource.z = Float2Fixed(zlen/length);
}



//---------------------- rotate light source -----------------------------


void RotateLightSource(void)
{
	short i;
	Fixedposhort nx,ny,nz;
	Fixedposhort cosxangle,sinxangle,cosyangle,sinyangle,coszangle,sinzangle;

	// update the light source rotation angles
	LightSource.ax+=8; if(LightSource.ax>=MAXDEGREES) LightSource.ax=0;
	LightSource.ay+=8; if(LightSource.ay>=MAXDEGREES) LightSource.ay=0;
	LightSource.az+=8; if(LightSource.az>=MAXDEGREES) LightSource.az=0;

	// get sine and cosine angles to save time from table lookup in inner loop
	sinxangle=sine[LightSource.ax];
	cosxangle=cosine[LightSource.ax];
	sinyangle=sine[LightSource.ay];
	cosyangle=cosine[LightSource.ay];
	sinzangle=sine[LightSource.az];
	coszangle=cosine[LightSource.az];

	// rotate around the x-axis
	LightSource.wz=FixedMul(LightSource.y , cosxangle) - FixedMul(LightSource.z , sinxangle);
	LightSource.wy=FixedMul(LightSource.y , sinxangle) + FixedMul(LightSource.z , cosxangle);
	LightSource.wx=LightSource.x;

	// rotate around the y-axis
	nx=FixedMul(LightSource.wx, cosyangle) - FixedMul(LightSource.wz , sinyangle);
	nz=FixedMul(LightSource.wx, sinyangle) + FixedMul(LightSource.wz , cosyangle);
	LightSource.wx=nx;
	LightSource.wz=nz;

	// rotate around the z-axis
	nx=FixedMul(LightSource.wx , coszangle) - FixedMul(LightSource.wy , sinzangle);
	ny=FixedMul(LightSource.wx , sinzangle) + FixedMul(LightSource.wy , coszangle);

	// reverse the direction of the normals for lightsource shading
	LightSource.wx=-nx;
	LightSource.wy=-ny;
	LightSource.wz=-nz;
}

//--------------------- Gouraud Shaded Triangle routine ------------------


void GTriangle(float x1,float y1,float c1,
							 float x2,float y2,float c2,
							 float x3,float y3,float c3,
							 short pageoffs)
{
	ulong adr;
	ulong col;
	float mx; // slope dy/dx
	float mc; // slope dc/dx
	float x; // x screen coordinate
	float c; // color of pixel to plot
	short count; // looping variable
	short y; // looping variable
	float cl; // color of left edge of horizontal scanline
	float cr; // color of right edge of horizontal scanline
	float el; // coordinate of left edge of horizontal scanline
	float er; // coordinate of right edge of horizontal scanline


	// initialize the edge buffers to extreme values
	for(count=0;count<240;count++)
	{
		edge_l[count]=319;
		edge_r[count]=0;
	}

	// scan the edges of the triangle
	ScanEdge(x1,y1,c1, x2,y2,c2);
	ScanEdge(x2,y2,c2, x3,y3,c3);
	ScanEdge(x3,y3,c3, x1,y1,c1);

	// gouraud fill the horizontal scanlines
	for(y=0;y<240;y++)
	{
		// if the scanline is valid (ie left coordinate < right coordinate)
		if(edge_l[y] <= edge_r[y])
		{
			cl=(float)color_l[y];
			cr=(float)color_r[y];
			el=(float)edge_l[y];
			er=(float)edge_r[y];

			// now calculate slope of color (dc/dx)

			if( (er-el) == 0)
			{
				// denominator will be zero so don't do the division so use dc
				mc = (cr-cl);
			}
			else
			{
				// denominator okay, so use dc/dx
				mc = (cr-cl)/(er-el);
			}
			c = cl; // start with color of left edge of scanline
			// loop through every pixel in horizontal scanline
			adr=edge_l[y]+(y*DISPLAY_X);
			for(count=(short)edge_l[y]; count < (short)edge_r[y]; count++)
			{
				col=((ulong)c*2)&0xFF;
				*(bits+adr)=colors[col];
				adr++;
				c+=mc; // increase the color by dc/dx
			}
		}
	}
}


void ScanEdge(float x1,float y1,float c1,float x2,float y2,float c2)
{
	// scan-convert (x1,y1,c1)-(x2,y2,c2)

	float mx; // slope dx/dy
	float mc; // slope dc/dy
	float x; // x pixel coordinate
	float c; // pixel's color
	short count; // looping variable
	float temp; // used for swapping

	// make sure that edge goes from top to bottom
	if( (y2-y1) < 0 )
	{
		// swap y2 with y1
		temp=y1;
		y1=y2;
		y2=temp;
		// swap x2 with x1
		temp=x1;
		x1=x2;
		x2=temp;
		// swap c2 with c1
		temp=c1;
		c1=c2;
		c2=temp;
	}

	// initialize for stepping
	if( (y2-y1) != 0)
	{
		mx = (x2-x1)/(y2-y1); // dx/dy
		mc = (c2-c1)/(y2-y1); // dc/dy
	}
	else
	{
		mx = (x2-x1); // dx
		mc = (c2-c1); // dc
	}
	x = x1; // starting x coordinate
	c = c1; // starting c color

	// step through edge and record color values along the way
	for(count=y1; count < y2; count++)
	{
		if(x < (float)edge_l[count])
		{
			edge_l[count]=(short)x;
			color_l[count]=(short)c;
		}
		if(x > (float)edge_r[count])
		{
			edge_r[count]=(short)x;
			color_r[count]=(short)c;
		}
		x+=mx;  // x = x + dx/dy
		c+=mc;  // c = c + dc/dy
	}
}


// Create application object and start it
short main(short argc, char **argv)
{	
	Gourad *the_app;

	srand(system_time());

	the_app = new Gourad();
	the_app->Run();
	delete the_app;

	return 0;
}


/*
 *  Application constructor
 */

Gourad::Gourad() : BApplication(APP_SIGNATURE)
{
	the_window = NULL;
	the_bitmap = NULL;
}

/*
 *	Thats from me!
 */

void Gourad::AboutRequested(void) 
{
	char str[256];
	sprintf(str, "Gourad by Tinic Urou\n<5uro@informatik.uni-hamburg.de>\nFreely distributable.");
	BAlert *the_alert = new BAlert("", str, "OK");
	the_alert->Go();
}


/*
 *  Open window and start rendering
 */

void Gourad::ReadyToRun(void)
{
	// Allocate the bitmap		
	my_bitmap = the_bitmap = new BBitmap(BRect(0, 0, DISPLAY_X-1, DISPLAY_Y-1), B_COLOR_8_BIT);
	bits=(uchar *)the_bitmap->Bits();

	for(int i=0;i<256;i++)
	{
		colors[i]=index_for_color(i+((rand())/(RAND_MAX/8)),i+((rand())/(RAND_MAX/8)),i+((rand())/(RAND_MAX/8)));
	}

	short y;
	char keypress;

	LoadObject();

	CreateLookupTables();

	SetupLightSource();
	CalculateNormals();

	// initialize the position and angles of object
	Object.Ox=Int2Fixed(0);
	Object.Oy=Int2Fixed(0);
	Object.Oz=Int2Fixed(-1024);
	Object.Ax=0;
	Object.Ay=0;
	Object.Az=0;

	// Open window
	the_window = new GouradWindow(DISPLAY_X, DISPLAY_Y, the_bitmap);

	// Start action
	the_window->PostMessage(MSG_REDRAW);

	// Let it flow from the orginal to the distorted look
}


/*
 *  Quit requested (either by menu or by closing the window)
 */

bool Gourad::QuitRequested(void)
{
	// Make sure that the window closes first
	if (BApplication::QuitRequested()) 
	{	
		if (the_bitmap)	delete the_bitmap;
		return TRUE;
	}
	return FALSE;
}


/*
 *  Window constructor
 */

GouradWindow::GouradWindow(short width, short height, BBitmap *bitmap) : BWindow(BRect(0, 0, width-1, height-1), "Gourad by Tinic", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	the_bitmap = bitmap;
	
	xmod = bitmap->BytesPerRow();
	
	// Move window to right position
	MoveTo(80, 60);

	// Create bitmap view
	Lock();
	main_view = new BView(BRect(0, 0, DISPLAY_X-1, DISPLAY_Y-1),"", B_FOLLOW_NONE,NULL);
	AddChild(main_view);
	Unlock();

	// Show the window
	Show();
}


/*
 *  Closing the window quits the program
 */

bool GouradWindow::QuitRequested(void)
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return TRUE;
}


/*
 *  Handles redraw messages
 */

void GouradWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case MSG_REDRAW:  // Redraw bitmap
			Lock();
			main_view->DrawBitmapAsync(the_bitmap);
			recalc();
			main_view->Sync();
			PostMessage(MSG_REDRAW);
			Unlock();
			break;
		default:
			BWindow::MessageReceived(msg);
	}
}


/*
 *  Main calculation
 */

void GouradWindow::recalc(void)
{
	UBYTE 	*q=(UBYTE *)the_bitmap->Bits();	
	
	Object.Ax+=8; if(Object.Ax >=MAXDEGREES) Object.Ax=0;
	Object.Ay-=8; if(Object.Ay <0) Object.Ay=MAXDEGREES-1;
	Object.Az+=6; if(Object.Az >=MAXDEGREES) Object.Az=0;
	RotateObject();
	RotateNormals();
	RotateLightSource();
	CalculateColor();
	MakePolygonList();
	SortPolygons();
	DrawPolygons();
}
